@@ -1,7 +1,7 @@
 /*
  * parser.c: parse a configuration file according to a grammar
  *
- * Copyright (C) 2007-2015 David Lutterkort
+ * Copyright (C) 2007-2016 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -68,7 +68,14 @@ struct state {
     uint                 nreg;
 };
 
-/* Used by recursive lenses to stack intermediate results */
+/* Used by recursive lenses to stack intermediate results
+   Frames are used for a few different purposes:
+
+   (1) to save results from individual lenses that are later gathered and
+       combined by combinators like L_STAR and L_CONCAT
+   (2) to preserve parse state when descending into a L_SUBTREE
+   (3) as a marker to determine if a L_MAYBE had a match or not
+ */
 struct frame {
     struct lens     *lens;
     char            *key;
@@ -108,6 +115,9 @@ struct rec_state {
     size_t               start;
     uint                 lvl;  /* Debug only */
     struct ast          *ast;
+    /* Will either be get_combine or parse_combine, depending on MODE, for
+       the duration of the whole recursive parse */
+    void (*combine)(struct rec_state *, struct lens *, uint);
 };
 
 #define REG_START(state) ((state)->regs->start[(state)->nreg])
@@ -119,6 +129,25 @@ struct rec_state {
 #define REG_MATCHED(state) (REG_VALID(state)                            \
                             && (state)->regs->start[(state)->nreg] >= 0)
 
+/* The macros SAVE_REGS and RESTORE_REGS are used to remember and restore
+ * where our caller was in processing their regexp match before we do
+ * matching ourselves (and would therefore clobber the caller's state)
+ *
+ * These two macros are admittedly horrible in that they declare local
+ * variables called OLD_REGS and OLD_NREG. The alternative to avoiding that
+ * would be to manually manage a stack of regs/nreg in STATE.
+ */
+#define SAVE_REGS(state)                                                \
+    struct re_registers *old_regs = state->regs;                        \
+    uint old_nreg = state->nreg;                                        \
+    state->regs = NULL;                                                 \
+    state->nreg = 0
+
+#define RESTORE_REGS(state)                                             \
+    free_regs(state);                                                   \
+    state->regs = old_regs;                                             \
+    state->nreg = old_nreg;
+
 /*
  * AST utils
  */
@@ -777,13 +806,11 @@ static struct tree *get_quant_star(struct lens *lens, struct state *state) {
     ensure0(lens->tag == L_STAR, state->info);
     struct lens *child = lens->child;
     struct tree *tree = NULL, *tail = NULL;
-    struct re_registers *old_regs = state->regs;
-    uint old_nreg = state->nreg;
     uint end = REG_END(state);
     uint start = REG_START(state);
     uint size = end - start;
 
-    state->regs = NULL;
+    SAVE_REGS(state);
     while (size > 0 && match(state, child, child->ctype, end, start) > 0) {
         struct tree *t = NULL;
 
@@ -794,9 +821,7 @@ static struct tree *get_quant_star(struct lens *lens, struct state *state) {
         size -= REG_SIZE(state);
         free_regs(state);
     }
-    free_regs(state);
-    state->regs = old_regs;
-    state->nreg = old_nreg;
+    RESTORE_REGS(state);
     if (size != 0) {
         short_iteration_error(lens, state, start, end);
     }
@@ -808,14 +833,12 @@ static struct skel *parse_quant_star(struct lens *lens, struct state *state,
     ensure0(lens->tag == L_STAR, state->info);
     struct lens *child = lens->child;
     struct skel *skel = make_skel(lens), *tail = NULL;
-    struct re_registers *old_regs = state->regs;
-    uint old_nreg = state->nreg;
     uint end = REG_END(state);
     uint start = REG_START(state);
     uint size = end - start;
 
     *dict = NULL;
-    state->regs = NULL;
+    SAVE_REGS(state);
     while (size > 0 && match(state, child, child->ctype, end, start) > 0) {
         struct skel *sk;
         struct dict *di = NULL;
@@ -828,9 +851,7 @@ static struct skel *parse_quant_star(struct lens *lens, struct state *state,
         size -= REG_SIZE(state);
         free_regs(state);
     }
-    free_regs(state);
-    state->regs = old_regs;
-    state->nreg = old_nreg;
+    RESTORE_REGS(state);
     if (size != 0) {
         get_error(state, lens, "%s", short_iteration);
     }
@@ -872,7 +893,7 @@ static struct skel *parse_quant_maybe(struct lens *lens, struct state *state,
 static struct tree *get_subtree(struct lens *lens, struct state *state) {
     char *key = state->key;
     char *value = state->value;
-    struct span *span = state->span;
+    struct span *span = move(state->span);
 
     struct tree *tree = NULL, *children;
 
@@ -887,10 +908,10 @@ static struct tree *get_subtree(struct lens *lens, struct state *state) {
 
     tree = make_tree(state->key, state->value, NULL, children);
     ERR_NOMEM(tree == NULL, state->info);
-    tree->span = state->span;
+    tree->span = move(state->span);
 
-    if (state->span != NULL) {
-        update_span(span, state->span->span_start, state->span->span_end);
+    if (tree->span != NULL) {
+        update_span(span, tree->span->span_start, tree->span->span_end);
     }
 
     state->key = key;
@@ -898,6 +919,8 @@ static struct tree *get_subtree(struct lens *lens, struct state *state) {
     state->span = span;
     return tree;
  error:
+    free_span(state->span);
+    state->span = span;
     return NULL;
 }
 
@@ -947,13 +970,12 @@ static struct tree *get_square(struct lens *lens, struct state *state) {
 
     struct lens *concat = lens->child;
     struct tree *tree = NULL;
-    struct re_registers *old_regs = state->regs;
-    uint old_nreg = state->nreg;
     uint end = REG_END(state);
     uint start = REG_START(state);
     char *rsqr = NULL, *lsqr = NULL;
     int r;
 
+    SAVE_REGS(state);
     r = match(state, lens->child, lens->child->ctype, end, start);
     ERR_NOMEM(r < 0, state->info);
 
@@ -982,9 +1004,7 @@ static struct tree *get_square(struct lens *lens, struct state *state) {
     }
 
  done:
-    free_regs(state);
-    state->nreg = old_nreg;
-    state->regs = old_regs;
+    RESTORE_REGS(state);
     FREE(lsqr);
     FREE(rsqr);
     return tree;
@@ -998,13 +1018,12 @@ static struct tree *get_square(struct lens *lens, struct state *state) {
 static struct skel *parse_square(struct lens *lens, struct state *state,
                                  struct dict **dict) {
     ensure0(lens->tag == L_SQUARE, state->info);
-    struct re_registers *old_regs = state->regs;
-    uint old_nreg = state->nreg;
     uint end = REG_END(state);
     uint start = REG_START(state);
     struct skel *skel = NULL, *sk = NULL;
     int r;
 
+    SAVE_REGS(state);
     r = match(state, lens->child, lens->child->ctype, end, start);
     ERR_NOMEM(r < 0, state->info);
 
@@ -1015,9 +1034,7 @@ static struct skel *parse_square(struct lens *lens, struct state *state,
     sk->skels = skel;
 
  error:
-    free_regs(state);
-    state->regs = old_regs;
-    state->nreg = old_nreg;
+    RESTORE_REGS(state);
     return sk;
 }
 
@@ -1036,7 +1053,11 @@ static void print_frames(struct rec_state *state) {
         } else {
             fprintf(stderr, " { %s = %s } ", f->tree->label, f->tree->value);
         }
-        fprintf(stderr, "%s\n", format_lens(f->lens));
+        char *s = format_lens(f->lens);
+        if (s != NULL) {
+            fprintf(stderr, "%s\n", s);
+            free(s);
+        }
     }
 }
 
@@ -1049,7 +1070,7 @@ static struct frame *top_frame(struct rec_state *state) {
 /* The nth frame from the top of the stack, where 0th frame is the top */
 ATTRIBUTE_PURE
 static struct frame *nth_frame(struct rec_state *state, uint n) {
-    assert(state->fsize > n);
+    ensure0(state->fsize > n, state->state->info);
     return state->frames + state->fused - (n+1);
 }
 
@@ -1078,11 +1099,9 @@ static struct frame *push_frame(struct rec_state *state, struct lens *lens) {
 static struct frame *pop_frame(struct rec_state *state) {
     ensure0(state->fused > 0, state->state->info);
 
+    struct frame *result = top_frame(state);
     state->fused -= 1;
-    if (state->fused > 0)
-        return top_frame(state);
-    else
-        return NULL;
+    return result;
 }
 
 static void dbg_visit(struct lens *lens, char action, size_t start, size_t end,
@@ -1117,17 +1136,17 @@ static void visit_terminal(struct lens *lens, size_t start, size_t end,
                            void *data) {
     struct rec_state *rec_state = data;
     struct state *state = rec_state->state;
-    struct re_registers *old_regs = state->regs;
     struct ast *child;
-    uint old_nreg = state->nreg;
 
     if (state->error != NULL)
         return;
 
+    SAVE_REGS(state);
     if (debugging("cf.get"))
         dbg_visit(lens, 'T', start, end, rec_state->fused, rec_state->lvl);
     match(state, lens, lens->ctype, end, start);
     struct frame *top = push_frame(rec_state, lens);
+    ERR_BAIL(state->info);
     if (rec_state->mode == M_GET)
         get_terminal(top, lens, state);
     else
@@ -1135,9 +1154,12 @@ static void visit_terminal(struct lens *lens, size_t start, size_t end,
     child = ast_append(rec_state, lens, start, end);
     ERR_NOMEM(child == NULL, state->info);
  error:
-    free_regs(state);
-    state->regs = old_regs;
-    state->nreg = old_nreg;
+    RESTORE_REGS(state);
+}
+
+static bool rec_gen_span(struct rec_state *rec_state) {
+    return ((rec_state->mode == M_GET)
+            && (rec_state->state->info->flags & AUG_ENABLE_SPAN));
 }
 
 static void visit_enter(struct lens *lens,
@@ -1156,18 +1178,24 @@ static void visit_enter(struct lens *lens,
     rec_state->lvl += 1;
     if (lens->tag == L_SUBTREE) {
         /* Same for parse and get */
+        /* Use this frame to preserve the current state before we process
+           the contents of the subtree, i.e., lens->child */
         struct frame *f = push_frame(rec_state, lens);
+        ERR_BAIL(state->info);
         f->key = state->key;
         f->value = state->value;
-        f->span = state->span;
         state->key = NULL;
         state->value = NULL;
-    } else if (lens->tag == L_MAYBE) {
-        push_frame(rec_state, lens);
-        if (state->info->flags & AUG_ENABLE_SPAN) {
+        if (rec_gen_span(rec_state)) {
+            f->span = state->span;
             state->span = make_span(state->info);
             ERR_NOMEM(state->span == NULL, state->info);
         }
+    } else if (lens->tag == L_MAYBE) {
+        /* Push a frame as a marker so we can tell whether lens->child
+           actually had a match or not */
+        push_frame(rec_state, lens);
+        ERR_BAIL(state->info);
     }
     child = ast_append(rec_state, lens, start, end);
     if (child != NULL)
@@ -1176,16 +1204,16 @@ static void visit_enter(struct lens *lens,
     return;
 }
 
+/* Combine n frames from the stack into one new frame for M_GET */
 static void get_combine(struct rec_state *rec_state,
                         struct lens *lens, uint n) {
     struct tree *tree = NULL, *tail = NULL;
     char *key = NULL, *value = NULL;
     struct frame *top = NULL;
 
-    if (n > 0)
-        top = top_frame(rec_state);
-
-    for (int i=0; i < n; i++, top = pop_frame(rec_state)) {
+    for (int i=0; i < n; i++) {
+        top = pop_frame(rec_state);
+        ERR_BAIL(lens->info);
         list_tail_cons(tree, tail, top->tree);
         /* top->tree might have more than one node, update tail */
         if (tail != NULL)
@@ -1201,6 +1229,7 @@ static void get_combine(struct rec_state *rec_state,
         }
     }
     top = push_frame(rec_state, lens);
+    ERR_BAIL(lens->info);
     top->tree = tree;
     top->key = key;
     top->value = value;
@@ -1208,6 +1237,7 @@ static void get_combine(struct rec_state *rec_state,
     return;
 }
 
+/* Combine n frames from the stack into one new frame for M_PUT */
 static void parse_combine(struct rec_state *rec_state,
                           struct lens *lens, uint n) {
     struct skel *skel = make_skel(lens), *tail = NULL;
@@ -1215,10 +1245,9 @@ static void parse_combine(struct rec_state *rec_state,
     char *key = NULL;
     struct frame *top = NULL;
 
-    if (n > 0)
-        top = top_frame(rec_state);
-
-    for (int i=0; i < n; i++, top = pop_frame(rec_state)) {
+    for (int i=0; i < n; i++) {
+        top = pop_frame(rec_state);
+        ERR_BAIL(lens->info);
         list_tail_cons(skel->skels, tail, top->skel);
         /* top->skel might have more than one node, update skel */
         if (tail != NULL)
@@ -1230,19 +1259,48 @@ static void parse_combine(struct rec_state *rec_state,
         }
     }
     top = push_frame(rec_state, lens);
-    top->skel = skel;
-    top->dict = dict;
+    ERR_BAIL(lens->info);
+    top->skel = move(skel);
+    top->dict = move(dict);
     top->key = key;
  error:
+    free_skel(skel);
+    free_dict(dict);
     return;
 }
 
+static void visit_exit_put_subtree(struct lens *lens,
+                                   struct rec_state *rec_state,
+                                   struct frame *top) {
+    struct state *state = rec_state->state;
+    struct skel *skel = NULL;
+    struct dict *dict = NULL;
+
+    skel = make_skel(lens);
+    ERR_NOMEM(skel == NULL, lens->info);
+    dict = make_dict(top->key, top->skel, top->dict);
+    ERR_NOMEM(dict == NULL, lens->info);
+
+    top = pop_frame(rec_state);
+    ERR_BAIL(state->info);
+    ensure(lens == top->lens, state->info);
+    state->key = top->key;
+    top = push_frame(rec_state, lens);
+    ERR_BAIL(state->info);
+    top->skel = move(skel);
+    top->dict = move(dict);
+ error:
+    free_skel(skel);
+    free_dict(dict);
+}
+
 static void visit_exit(struct lens *lens,
                        ATTRIBUTE_UNUSED size_t start,
                        ATTRIBUTE_UNUSED size_t end,
                        void *data) {
     struct rec_state *rec_state = data;
     struct state *state = rec_state->state;
+    struct tree *tree = NULL;
 
     if (state->error != NULL)
         return;
@@ -1254,40 +1312,32 @@ static void visit_exit(struct lens *lens,
     ERR_BAIL(lens->info);
 
     if (lens->tag == L_SUBTREE) {
-        struct frame *top = top_frame(rec_state);
+        /* Get the result of parsing lens->child */
+        struct frame *top = pop_frame(rec_state);
+        ERR_BAIL(state->info);
         if (rec_state->mode == M_GET) {
-            struct tree *tree;
-            // FIXME: tree may leak if pop_frame ensure0 fail
             tree = make_tree(top->key, top->value, NULL, top->tree);
-            tree->span = state->span;
             ERR_NOMEM(tree == NULL, lens->info);
+            tree->span = state->span;
+            /* Restore the parse state from before entering this subtree */
             top = pop_frame(rec_state);
+            ERR_BAIL(state->info);
             ensure(lens == top->lens, state->info);
             state->key = top->key;
             state->value = top->value;
             state->span = top->span;
-            pop_frame(rec_state);
+            /* Push the result of parsing this subtree */
             top = push_frame(rec_state, lens);
-            top->tree = tree;
+            ERR_BAIL(state->info);
+            top->tree = move(tree);
         } else {
-            struct skel *skel;
-            struct dict *dict;
-            skel = make_skel(lens);
-            ERR_NOMEM(skel == NULL, lens->info);
-            dict = make_dict(top->key, top->skel, top->dict);
-            ERR_NOMEM(dict == NULL, lens->info);
-            top = pop_frame(rec_state);
-            ensure(lens == top->lens, state->info);
-            state->key = top->key;
-            pop_frame(rec_state);
-            top = push_frame(rec_state, lens);
-            top->skel = skel;
-            top->dict = dict;
+            visit_exit_put_subtree(lens, rec_state, top);
         }
     } else if (lens->tag == L_CONCAT) {
         ensure(rec_state->fused >= lens->nchildren, state->info);
         for (int i = 0; i < lens->nchildren; i++) {
             struct frame *fr = nth_frame(rec_state, i);
+            ERR_BAIL(state->info);
             BUG_ON(lens->children[i] != fr->lens,
                     lens->info,
              "Unexpected lens in concat %zd..%zd\n  Expected: %s\n  Actual: %s",
@@ -1295,29 +1345,26 @@ static void visit_exit(struct lens *lens,
                     format_lens(lens->children[i]),
                     format_lens(fr->lens));
         }
-        if (rec_state->mode == M_GET)
-            get_combine(rec_state, lens, lens->nchildren);
-        else
-            parse_combine(rec_state, lens, lens->nchildren);
+        rec_state->combine(rec_state, lens, lens->nchildren);
     } else if (lens->tag == L_STAR) {
         uint n = 0;
         while (n < rec_state->fused &&
                nth_frame(rec_state, n)->lens == lens->child)
             n++;
-        if (rec_state->mode == M_GET)
-            get_combine(rec_state, lens, n);
-        else
-            parse_combine(rec_state, lens, n);
+        ERR_BAIL(state->info);
+        rec_state->combine(rec_state, lens, n);
     } else if (lens->tag == L_MAYBE) {
         uint n = 1;
         if (rec_state->fused > 0
             && top_frame(rec_state)->lens == lens->child) {
             n = 2;
         }
-        if (rec_state->mode == M_GET)
-            get_combine(rec_state, lens, n);
-        else
-            parse_combine(rec_state, lens, n);
+        ERR_BAIL(state->info);
+        /* If n = 2, the top of the stack is our child's result, and the
+           frame underneath it is the marker frame we pushed during
+           visit_enter. Combine these two frames into one, which represents
+           the result of parsing the whole L_MAYBE. */
+        rec_state->combine(rec_state, lens, n);
     } else if (lens->tag == L_SQUARE) {
         if (rec_state->mode == M_GET) {
             struct ast *square, *concat, *right, *left;
@@ -1340,15 +1387,17 @@ static void visit_exit(struct lens *lens,
             FREE(rsqr);
             if (! ret)
                 goto error;
-            get_combine(rec_state, lens, 1);
-        } else {
-            parse_combine(rec_state, lens, 1);
         }
+        rec_state->combine(rec_state, lens, 1);
     } else {
+        /* Turn the top frame from having the result of one of our children
+           to being our result */
         top_frame(rec_state)->lens = lens;
+        ERR_BAIL(state->info);
     }
     ast_pop(rec_state);
  error:
+    free_tree(tree);
     return;
 }
 
@@ -1368,8 +1417,6 @@ static struct frame *rec_process(enum mode_t mode, struct lens *lens,
     uint end = REG_END(state);
     uint start = REG_START(state);
     size_t len = 0;
-    struct re_registers *old_regs = state->regs;
-    uint old_nreg = state->nreg;
     int r;
     struct jmt_visitor visitor;
     struct rec_state rec_state;
@@ -1378,21 +1425,20 @@ static struct frame *rec_process(enum mode_t mode, struct lens *lens,
 
     MEMZERO(&rec_state, 1);
     MEMZERO(&visitor, 1);
+    SAVE_REGS(state);
 
     if (lens->jmt == NULL) {
         lens->jmt = jmt_build(lens);
         ERR_BAIL(lens->info);
     }
 
-    state->regs = NULL;
-    state->nreg = 0;
-
     rec_state.mode  = mode;
     rec_state.state = state;
     rec_state.fused = 0;
     rec_state.lvl   = 0;
     rec_state.start = start;
     rec_state.ast = make_ast(lens);
+    rec_state.combine = (mode == M_GET) ? get_combine : parse_combine;
     ERR_NOMEM(rec_state.ast == NULL, state->info);
 
     visitor.parse = jmt_parse(lens->jmt, state->text + start, end - start);
@@ -1423,8 +1469,7 @@ static struct frame *rec_process(enum mode_t mode, struct lens *lens,
  done:
     if (debugging("cf.get.ast"))
         print_ast(ast_root(rec_state.ast), 0);
-    state->regs = old_regs;
-    state->nreg = old_nreg;
+    RESTORE_REGS(state);
     jmt_free_parse(visitor.parse);
     free_ast(ast_root(rec_state.ast));
     return rec_state.frames;
@@ -1433,6 +1478,7 @@ static struct frame *rec_process(enum mode_t mode, struct lens *lens,
     for(i = 0; i < rec_state.fused; i++) {
         f = nth_frame(&rec_state, i);
         FREE(f->key);
+        free_span(f->span);
         if (mode == M_GET) {
             FREE(f->value);
             free_tree(f->tree);
