@@ -1,7 +1,7 @@
 /*
  * internal.c: internal data structures and helpers
  *
- * Copyright (C) 2007-2015 David Lutterkort
+ * Copyright (C) 2007-2016 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -354,9 +354,7 @@ char *path_expand(struct tree *tree, const char *ppath) {
     if (ppath == NULL)
         ppath = "";
 
-    if (tree == NULL)
-        label = "(no_tree)";
-    else if (tree->label == NULL)
+    if (tree->label == NULL)
         label = "(none)";
     else
         label = tree->label;
